#include "net.h"

/********************************** basic net operations **********************************/

void
init_net(net_t* net)
/*randomly initializes the neural network received as argument */
{
	int	i, j;

	srand((time_t)time(NULL));
	
	for(i = 0; i < NR_INPUT_NODES; i++)
		for(j = 0; j < NR_HIDDEN_NODES; j++)
			net->i2h[i][j] = (float)((rand() + 0.0) / RAND_MAX);

	for(i = 0; i < NR_HIDDEN_NODES; i++)
		net->h2o[i] = (float)((rand() + 0.0) / RAND_MAX);

	net->alpha = (float)ALPHA;
}

void
activate_net(net_t* net)
/*activates the net given the input values, and returns the network output*/
{
	int		i, j;

	net->response = 0;
	memset(net->hidden, 0, sizeof(net->hidden));
	for(i = 0; i < NR_HIDDEN_NODES; i++)
	{
		for(j = 0; j < NR_INPUT_NODES; j++)
		{
			net->hidden[i] += net->i2h[j][i] * net->input[j];
		}
		net->hidden[i] = TANH(net->hidden[i]);
	}

	for(i = 0; i < NR_HIDDEN_NODES; i++)
		net->response += net->hidden[i] * net->h2o[i];
	net->response = TANH(net->response);	
}

void
learn(net_t* net, float error)
/*updates the weights of the network, according to the error and the learning rate*/
{
	int	i, j;
	float	update = net->alpha * error;

	for(i = 0; i < NR_INPUT_NODES; i++)
		for(j = 0; j < NR_HIDDEN_NODES; j++)
			net->i2h[i][j] += update;

	for(i = 0; i < NR_HIDDEN_NODES; i++)
		net->h2o[i] += update;
}

/************************************ disk operations *************************************/

void
save_net(net_t* net, char* file)
{
	int	fd;

	if((fd = open(file, O_CREAT | O_TRUNC | O_RDONLY, S_IREAD | S_IWRITE)) < 0)
	{
		printf("Can't create net-file\n");
		exit(100);
	}

	write(fd, &net->alpha, sizeof(net->alpha));
	write(fd, &net->i2h, sizeof(net->h2o));
	write(fd, &net->h2o, sizeof(net->h2o));

	close(fd);
}

void
load_net(net_t* net, char* file)
{
	int	fd;

	if((fd = open(file, O_RDONLY)) < 0)
	{
		printf("Can't create net-file\n");
		exit(100);
	}

	read(fd, &net->alpha, sizeof(net->alpha));
	read(fd, &net->i2h, sizeof(net->h2o));
	read(fd, &net->h2o, sizeof(net->h2o));

	close(fd);
}

