#include "tdnet.h"


/********************************* basic tdnet operations *********************************/

void
init_tdnet(tdnet_t* net)
/*randomly initializes the neural network received as argument */
{
	int	i, j;

	srand((time_t)time(NULL));
	
	for(i = 0; i < NR_INPUT_NODES; i++)
	{
		for(j = 0; j < NR_HIDDEN_NODES; j++)
		{

			net->i2h[i][j]	= (float)((rand() + 0.0) / RAND_MAX);
			net->ei2h[i][j]	= 0.;
		}
	}

	for(i = 0; i < NR_HIDDEN_NODES; i++)
	{
		net->h2o[i]		= (float)((rand() + 0.0) / RAND_MAX);
		net->eh2o[i]	= 0.;
	}

	net->alpha	= (float)TDALPHA;
	net->beta	= (float)TDBETA;
	net->gamma	= (float)GAMMA;
	net->lambda	= (float)LAMBDA;
}

void
activate_tdnet(tdnet_t* net)
/*activates the net given the input values, and returns the network output*/
{
	int		i, j;

	net->response = 0;
	memset(net->hidden, 0, sizeof(net->hidden));

	for(i = 0; i < NR_HIDDEN_NODES; i++)
	{
		for(j = 0; j < NR_INPUT_NODES; j++)
			net->hidden[i] += net->input[j] * net->i2h[j][i];
		net->hidden[j] = TANH(net->hidden[j]);		/* hyperbolic tangent */
	}

	for (i = 0; i < NR_HIDDEN_NODES; i++)
		net->response += net->hidden[i] * net->h2o[i];
	net->response = TANH(net->response); /* hyperbolic tangent */
}

void
tdlearn(tdnet_t* net, float error)
/* online updating */
{
	int i, j;

	for (i = 0; i < NR_HIDDEN_NODES; i++)
		for (j = 0; j < NR_INPUT_NODES; j++)
			net->i2h[j][i] += net->alpha * error * net->ei2h[j][i];

	for (i = 0; i < NR_HIDDEN_NODES; i++)
		net->h2o[i] += net->beta * error * net->eh2o[i];
}

void
update_eligibilities(tdnet_t* net)
{
	int		i,j;
	float	temp;

	temp = net->response * (1 - net->response);

	for (i = 0; i < NR_HIDDEN_NODES; i++)
		for(j = 0; j < NR_INPUT_NODES; j++)
			net->ei2h[j][i] = net->lambda * net->ei2h[j][i] +
									temp * net->h2o[i] * net->hidden[i] *
									(1 - net->hidden[i]) * net->input[j];

	for(i = 0; i < NR_HIDDEN_NODES; i++)
		net->eh2o[i] = net->lambda * net->eh2o[j] + temp * net->hidden[i];

}

/************************************ disk operations *************************************/

void
save_tdnet(tdnet_t* net, char* file)
{}

void
load_tdnet(tdnet_t* net, char* file)
{}